/* wrs_umon_server.c : white rabbit switch micro monitor - server
*
*  author: Dietrich Beck, GSI-Darmstadt
*/
#define VERSIONSTR  "0.01b"
/*
*  history: This program was developed as a exercise. The main task is to provide
*  very basic monitoring of a White Rabbit switch. The communication layer is DIM.
*  
*  The code retrieving information from the servo and the ports has been taken from
*  wr_mon, a command line program included in the White Rabbit switch software 3.1.
*  30-JAN-2012 first version
*
*  purpose: publish WR switch status
*
*  License Agreement for this software:
*
*  Copyright (C)
*  GSI Helmholtzzentrum für Schwerionenforschung GmbH
*  Planckstr. 1
*  64291 Darmstadt
*  Germany
*
*  Contact: D.Beck@gsi.de 
*  
*  This program is free software; you can redistribute it and/or modify it under the terms of the 
*  GNU General Public License as published by the Free Software Foundation; either version 2 of 
*  the license, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
*  the GNU General License for more details (http://www.gnu.org).
*  
*  GSI Helmholtzzentrum für Schwerionenforschung GmbH
*  Planckstr. 1, 64291 Darmstadt, Germany
*  For all questions and ideas contact: D.Beck@gsi.de
*  Last update: 31-JAN-2012
*/

/* standard includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* WR includes */
#include <minipc.h>
#define PTP_EXPORT_STRUCTURES
#include "ptpd_exports.h"
#include "hal_client.h"

/* DIM includes */
#include <dis.h>

/* umon constants */
#define EMPTYSTR_S  "        "
#define EMPTYSTR_L  "                                "
#define SERVLEN_S 9  // reserve space for one extra byte to be sure...
#define SERVLEN_L 33
#define MAXLEN 132

/* WRS stuff */
hexp_port_list_t port_list;
static struct minipc_ch *ptp_ch;

/* DIM service IDs */
int versionID = -1;
int syncStateID = -1;
int portStateID = -1; 
int myNameID = -1;

/* DIM services */ 
char version[SERVLEN_S];
char syncState[SERVLEN_S];
char portState[SERVLEN_L];
char myName[SERVLEN_L];

/*----------------------------------------------------------------
  message : print a message to stdout
  ----------------------------------------------------------------*/
void message(char *msg)
{
  char buff[MAXLEN];

  sprintf(buff, "%s - umon: %s\n", myName, msg);
  printf(buff);

} /* message */

/*----------------------------------------------------------------
  wrs_init: initialize stuff to obtain info from the switch 
  ----------------------------------------------------------------*/
void wrs_init()
{
	halexp_client_init();

	ptp_ch = minipc_client_create("ptpd", 0);
	if (!ptp_ch)
	{
		fprintf(stderr,"Can't establish WRIPC connection "
			"to the PTP daemon!\n");
		exit(-1);
	}
	halexp_query_ports(&port_list);
} /* wrs_init */

/*----------------------------------------------------------------
  get_port_state: get info on switch ports
  ----------------------------------------------------------------*/
void get_port_state()
{
	int i, j;
	char buff[SERVLEN_L];
	char temp[8];
	char mode[8];

	sprintf(buff, "");

	for(i=0; i<18;i++)
	{
		char if_name[10], found = 0;
		hexp_port_state_t state;
		short up;

		snprintf(if_name, 10, "wr%d", i);
		sprintf(mode, "E");
		
		for(j=0;j<port_list.num_ports;j++)
			if(!strcmp(port_list.port_names[j], if_name)) { found = 1; break; }
			
		if(!found) continue;
		
		halexp_get_port_state(&state, if_name);

		if(state.up) up = 1; else up = 0;

		switch(state.mode)
		{
			case HEXP_PORT_MODE_WR_MASTER:
			  sprintf(mode, "M");
			  break;
			case HEXP_PORT_MODE_WR_SLAVE:
			  sprintf(mode, "S");
			  break;
		}
		if (up) sprintf(temp, "%s", mode);
		else  sprintf(temp, "-");
		strcat(buff, temp); 
	} /* for i */
	
	sprintf(portState, "%s", EMPTYSTR_L);
	sprintf(portState, "%s", buff);
} /* get port state */

/*----------------------------------------------------------------
  get_sync_state: get info on WR sync state
  ----------------------------------------------------------------*/

void get_sync_state()
{
    ptpdexp_sync_state_t ss;

	minipc_call(ptp_ch, 2000, &__rpcdef_get_sync_state, &ss);

    sprintf(syncState, "%s", EMPTYSTR_S);

	if(!ss.valid) sprintf(syncState, "NO_SYNC");
	else sprintf(syncState, "%s", ss.sync_source);
} /* get_sync_state */

/*----------------------------------------------------------------
  add_dim_service: declare DIM services published by this server
  ----------------------------------------------------------------*/
void add_dim_services(char *serverID)
{
  char   buff[MAXLEN];

  //version number
  sprintf(buff, "%s/wrs_umon/version", serverID);
  sprintf(version, "%s", VERSIONSTR);
  versionID = dis_add_service(buff, "C", version, SERVLEN_S, 0, 0);

  //sync state
  sprintf(buff, "%s/wrs_umon/syncState", serverID);
  sprintf(syncState,"%s", EMPTYSTR_S);
  sprintf(syncState,"%s", "UNKNOWN");
  syncStateID = dis_add_service(buff, "C", syncState, SERVLEN_S, 0, 0);

  //port state
  sprintf(buff, "%s/wrs_umon/portState", serverID);
  sprintf(portState,"%s", EMPTYSTR_L);
  sprintf(portState,"%s", "UNKNOWN");
  portStateID = dis_add_service(buff, "C", portState, SERVLEN_L, 0, 0);

  //wrsmon ID
  sprintf(buff, "%s/wrs_umon/ID", serverID);
  sprintf(myName, "%s", EMPTYSTR_L);
  sprintf(myName, "%s", serverID);
  myNameID = dis_add_service(buff, "C", myName, SERVLEN_L, 0, 0);
} /* add_dim_services */


main(int argc, char **argv)
{
  char   buff[MAXLEN];
  char   oldSyncState[SERVLEN_S];
  char   oldPortState[SERVLEN_L];

  sprintf(buff, "white rabbit switch micro monitor: server version %s\n", VERSIONSTR);
  printf(buff);

  if (argc < 2) 
	{
	  printf("Usage: wrs_umon_server IDENTIFIER\n");
	  printf("IDENTIFIER must be a unique identifier like IP number or hostname.\n");
	  exit(1);
	}

  //init some strings
  sprintf(oldSyncState, "%s", EMPTYSTR_L);
  sprintf(oldPortState, "%s", EMPTYSTR_L);

  
  //start DIM server
  add_dim_services(argv[1]);
  sprintf(buff, "WRS_UMON_%s", myName);
  dis_start_serving(buff);

  wrs_init();
  for(;;)
  {
	usleep(500003);

	get_sync_state();
	get_port_state();

	if (strcmp(syncState, oldSyncState))
	{
	  /* update sync state */
	  dis_update_service(syncStateID);
	  sprintf(oldSyncState, "%s", syncState);
	  sprintf(buff, "sync state: %s", syncState);
	  message(buff);
	}

	if (strcmp(portState, oldPortState))
	{
	  /* update port state */
	  dis_update_service(portStateID);
	  sprintf(oldPortState, "%s", portState);
	  sprintf(buff, "port state: %s", portState);
	  message(buff);
	}
  } /* for ;;; */
} /* main */
